/*
 *	NEWS-0.4.4 - RSS Feed Ticker - Copyright (C) Emmanuel Thomas-Maurin 2009-2010
 *	< manutm007@gmail.com >
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news.h"

#ifdef G_OS_WIN32
extern FILE	*stdout_fp, *stderr_fp;
#endif

/*
 * (re)load text file if it exists or rss feed if valid
 */
int load_resource(Resource *resrc, int *rss_ttl)
{
#ifdef G_OS_WIN32
	gchar	*win32_filename;
#endif
	int	exit_status;

	if (resrc->fp != NULL) {
		fclose(resrc->fp);
		resrc->fp = NULL;
	}
	resrc->xmldump[0] = '\0';

	if (resrc->id[0] == '\0') {
		fprintf(STD_ERR, "No resource specified\n");
		exit_status = RESOURCE_UNSPECIFIED;

	} else {
	/*
	 * is there an existing file?
	 */
#ifndef G_OS_WIN32
		if ((resrc->fp = g_fopen(resrc->id, "rb")) == NULL) {
#else
		win32_filename = g_win32_locale_filename_from_utf8(resrc->id);
		if ((resrc->fp = g_fopen(win32_filename, "rb")) == NULL) {
			g_free(win32_filename);
#endif
			/*
			 * if not, have we a valid rss url instead?
			 */
			if (get_rss_feed(resrc, get_params(), rss_ttl) != RSS_OK) {
				warning("load_resource():", "invalid resource:", resrc->id, "");
				resrc->id[0] ='\0';
				exit_status = RESOURCE_INVALID;
			} else
				exit_status = RESOURCE_OK;
		} else {
#ifdef G_OS_WIN32
			g_free(win32_filename);
#endif
			exit_status = RESOURCE_OK;
		}
	}

	if (exit_status == RESOURCE_OK) {
		if (format_resource(resrc) == RESOURCE_FORMAT_ERROR) {
			big_error(RESOURCE_FORMAT_ERROR,
				"format_resource() error", "", "", "");
			exit_status = RESOURCE_FORMAT_ERROR;
		}
	}
	return exit_status;
}

/*
 * check/convert to utf8
 * then check for lines longer than LINE_MAXLEN and insert newline's if any
 * then resrc->fp refers to a "formatted" file
 */
int format_resource(Resource *resrc)
{
	FILE		*tmp_fp;
	char		tmp_file_name[TMPSTR_SIZE + 1];
#ifdef G_OS_WIN32
	gchar		*win32_filename;
#endif
	char		*str;
	size_t		str_size = 1024;
	gchar		*utf8_str = NULL;
	gsize		bytes_read, bytes_written;
	GError		*error = NULL;
	unsigned int	c;
	int		i = 0;

#ifndef G_OS_WIN32
	snprintf(tmp_file_name, TMPSTR_SIZE - 1, "%s/%s", usr_home_dir(), TMP_XMLDUMP);
	if (get_instance_id() != 0)
		str_n_cat(tmp_file_name, itoa2(get_instance_id()), 2);
	tmp_fp = g_fopen(tmp_file_name, "wb+");
#else
	snprintf(tmp_file_name, TMPSTR_SIZE - 1, "%s\\%s", get_appdata_dir(), TMP_XMLDUMP);
	if (get_instance_id() != 0)
		str_n_cat(tmp_file_name, itoa2(get_instance_id()), 2);
	win32_filename = g_win32_locale_filename_from_utf8(tmp_file_name);
	tmp_fp = g_fopen(win32_filename, "wb+");
	g_free(win32_filename);
#endif
	if (tmp_fp == NULL) {
		big_error(RESOURCE_FORMAT_ERROR, "Error creating file:", tmp_file_name, " - ", strerror(errno));
		return RESOURCE_FORMAT_ERROR;
	}

	/* check/convert to utf8 */
	str = malloc2(str_size * sizeof(char));
#ifndef G_OS_WIN32
	while (getline(&str, &str_size, resrc->fp) != -1) {
#else
	while (fgets(str, str_size, resrc->fp) != NULL) {
#endif
		if (!g_utf8_validate((const gchar *)str, -1, NULL)) {
			fprintf(STD_ERR, "Non UTF-8 encoded string: ");
			if ((utf8_str = g_locale_to_utf8((const gchar*)str, -1, &bytes_read, &bytes_written, &error)) != NULL) {
				fprintf(STD_ERR, "fixed\n");
				fputs(utf8_str, tmp_fp);
			} else {
				fprintf(STD_ERR, "re-encoding failed: %s\n", error->message);
				g_error_free(error);
			}
		} else
			fputs(str, tmp_fp);
	}
	free2(str);
	fclose(resrc->fp);
	resrc->fp = tmp_fp;
	fseek(resrc->fp, 0, SEEK_SET);

#ifndef G_OS_WIN32
	snprintf(tmp_file_name, TMPSTR_SIZE - 1, "%s/%s", usr_home_dir(), XMLDUMP);
	if (get_instance_id() != 0)
		str_n_cat(tmp_file_name, itoa2(get_instance_id()), 2);
	tmp_fp = g_fopen(tmp_file_name, "wb+");
#else
	snprintf(tmp_file_name, TMPSTR_SIZE - 1, "%s\\%s", get_appdata_dir(), XMLDUMP);
	if (get_instance_id() != 0)
		str_n_cat(tmp_file_name, itoa2(get_instance_id()), 2);
	win32_filename = g_win32_locale_filename_from_utf8(tmp_file_name);
	tmp_fp = g_fopen(win32_filename, "wb+");
	g_free(win32_filename);
#endif
	if (tmp_fp == NULL) {
		big_error(RESOURCE_FORMAT_ERROR, "Error creating file:", tmp_file_name, " - ", strerror(errno));
		return RESOURCE_FORMAT_ERROR;
	}

	while ((c = fgetc(resrc->fp)) != (unsigned int)EOF) {
		if (c == (unsigned int)'\n')
			i = 0;
		/*
		 * if the word we are parsing isn't longer than WORD_MAXLEN
		 * chars, we replace the 1st ' ' found after it with a '\n'
		 * otherwise the word is cut
		 */
		if (i >= LINE_MAXLEN - WORD_MAXLEN)
			if (i >= LINE_MAXLEN || c == (unsigned int)' ') {
				fputc((unsigned int)'\n', tmp_fp);
				i = 0;
				continue;
			}
		fputc((unsigned int)c, tmp_fp);
		i++;
	}
	fclose(resrc->fp);
	resrc->fp = tmp_fp;
	fseek(resrc->fp, 0, SEEK_SET);

	return RESOURCE_OK;
}
